<?php

namespace App\Tests\functional;

use App\Entity\Capsule;
use App\Entity\Group;
use App\Entity\User;
use App\Repository\CapsuleRepository;
use App\Repository\GroupRepository;
use App\Repository\UserRepository;
use Doctrine\Persistence\ObjectManager;
use Symfony\Bundle\FrameworkBundle\KernelBrowser;
use Symfony\Bundle\FrameworkBundle\Test\WebTestCase;

class CapsuleGroupControllerTest extends WebTestCase
{
    private KernelBrowser $client;
    private ObjectManager $object_manager;
    private UserRepository $user_repository; /** @phpstan-ignore-line */
    private CapsuleRepository $capsule_repository; /** @phpstan-ignore-line */
    private GroupRepository $group_repository;
    private User $user_1;
    private User $user_2;
    private Group $group_pop_own_by_user_1;
    private Group $group_rock_own_by_user_1;

    public function testUserShouldNotBeAbleToCreateACapsuleGroupWithExistingGroupName(): void
    {
        $this->client->loginUser($this->user_1);
        $uri = '/en/my_groups/create';
        $crawler = $this->client->request('GET', $uri);
        $this->assertResponseIsSuccessful();

        $this->client->enableProfiler();
        $submit_button = $crawler->selectButton('Create a new group');
        $form = $submit_button->form();
        $this->assertContains($this->group_pop_own_by_user_1, $this->user_1->getGroups()->toArray());

        $form['create_capsule_groups_form[name]'] = $this->group_pop_own_by_user_1->getName();
        $this->client->submit($form);

        $this->assertFalse($this->client->getResponse()->isRedirect());
    }

    public function testUserShouldBeAbleToCreateACapsuleGroupWithTheSameGroupNameUsedByAnotherUser(): void
    {
        $this->client->loginUser($this->user_2);
        $uri = '/en/my_groups/create';
        $crawler = $this->client->request('GET', $uri);
        $this->assertResponseIsSuccessful();

        $this->client->enableProfiler();
        $submit_button = $crawler->selectButton('Create a new group');
        $form = $submit_button->form();
        $this->assertContains($this->group_pop_own_by_user_1, $this->user_1->getGroups()->toArray());

        $form['create_capsule_groups_form[name]'] = $this->group_rock_own_by_user_1->getName();
        $this->client->submit($form);

        $this->assertTrue($this->client->getResponse()->isRedirect());
    }

    protected function setUp(): void
    {
        self::ensureKernelShutdown();

        $this->client = static::createClient();

        $this->object_manager = $this->client->getContainer()
            ->get('doctrine')
            ->getManager();

        $this->user_repository = $this->object_manager->getRepository(User::class);
        $this->capsule_repository = $this->object_manager->getRepository(Capsule::class);
        $this->group_repository = $this->object_manager->getRepository(Group::class);

        $this->setUsers();
        $this->setCapsuleGroupPop();
        $this->setCapsuleGroupRock();
    }

    private function setUsers(): void
    {
        $verified_user_1 = $this->user_repository
            ->findOneBy(['email' => 'defaultUser@localhost.com']);

        if (! $verified_user_1 instanceof User) {
            throw new \Exception("User does not exist.");
        }

        $this->user_1 = $verified_user_1;

        $user_2 = $this->user_repository
            ->findOneBy(['email' => 'defaultUser2@localhost.com']);

        if (! $user_2 instanceof User) {
            throw new \Exception("User does not exist.");
        }

        $this->user_2 = $user_2;
    }

    private function setCapsuleGroupPop(): void
    {
        $group_pop = $this->group_repository->findOneBy(['name' => 'Pop', 'author' => $this->user_1]);

        if (! $group_pop instanceof Group) {
            throw new \Exception("Group does not exist.");
        }

        $this->group_pop_own_by_user_1 = $group_pop;
    }

    private function setCapsuleGroupRock(): void
    {
        $group_rock = $this->group_repository->findOneBy(['name' => 'Rock', 'author' => $this->user_1]);

        if (! $group_rock instanceof Group) {
            throw new \Exception("Group does not exist.");
        }

        $this->group_rock_own_by_user_1 = $group_rock;
    }

    protected function tearDown(): void
    {
        parent::tearDown();
        self::ensureKernelShutdown();
    }
}
